@@ -1,17 +1,20 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Description: [MS-TDS] & [MC-SQLR] example.
+# Description:
+#   [MS-TDS] & [MC-SQLR] example.
 #
 # Author:
-#  Alberto Solino (beto@coresecurity.com/@agsolino)
+#   Alberto Solino (@agsolino)
 #
 # Reference for:
-#  Structure
+#   Structure
 #
 
 from __future__ import division
@@ -22,6 +25,7 @@ import os
 import logging
 
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket import version, tds
 
 if __name__ == '__main__':
@@ -149,15 +153,7 @@ if __name__ == '__main__':
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
-
-    domain, username, password, address = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-    
-    #In case the password contains '@'
-    if '@' in address:
-        password = password + '@' + address.rpartition('@')[0]
-        address = address.rpartition('@')[2]
+    domain, username, password, address = parse_target(options.target)
 
     if domain is None:
         domain = ''
