@@ -1,35 +1,37 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Generic NTLM Relay Module
+# Description:
+#   Generic NTLM Relay Module
+#
+#   This module performs the SMB Relay attacks originally discovered
+#   by cDc extended to many target protocols (SMB, MSSQL, LDAP, etc).
+#   It receives a list of targets and for every connection received it
+#   will choose the next target and try to relay the credentials. Also, if
+#   specified, it will first to try authenticate against the client connecting
+#   to us.
+#
+#   It is implemented by invoking a SMB and HTTP Server, hooking to a few
+#   functions and then using the specific protocol clients (e.g. SMB, LDAP).
+#   It is supposed to be working on any LM Compatibility level. The only way
+#   to stop this attack is to enforce on the server SPN checks and or signing.
+#
+#   If the authentication against the targets succeeds, the client authentication
+#   succeeds as well and a valid connection is set against the local smbserver.
+#   It's up to the user to set up the local smbserver functionality. One option
+#   is to set up shares with whatever files you want to so the victim thinks it's
+#   connected to a valid SMB server. All that is done through the smb.conf file or
+#   programmatically.
 #
 # Authors:
-#  Alberto Solino (@agsolino)
-#  Dirk-jan Mollema / Fox-IT (https://www.fox-it.com)
-#
-# Description:
-#             This module performs the SMB Relay attacks originally discovered
-# by cDc extended to many target protocols (SMB, MSSQL, LDAP, etc).
-# It receives a list of targets and for every connection received it
-# will choose the next target and try to relay the credentials. Also, if
-# specified, it will first to try authenticate against the client connecting
-# to us.
-#
-# It is implemented by invoking a SMB and HTTP Server, hooking to a few
-# functions and then using the specific protocol clients (e.g. SMB, LDAP).
-# It is supposed to be working on any LM Compatibility level. The only way
-# to stop this attack is to enforce on the server SPN checks and or signing.
-#
-# If the authentication against the targets succeeds, the client authentication
-# succeeds as well and a valid connection is set against the local smbserver.
-# It's up to the user to set up the local smbserver functionality. One option
-# is to set up shares with whatever files you want to so the victim thinks it's
-# connected to a valid SMB server. All that is done through the smb.conf file or
-# programmatically.
+#   Alberto Solino (@agsolino)
+#   Dirk-jan Mollema / Fox-IT (https://www.fox-it.com)
 #
 
 import argparse
@@ -162,6 +164,8 @@ def start_servers(options, threads):
         c.setInterfaceIp(options.interface_ip)
         c.setExploitOptions(options.remove_mic, options.remove_target)
         c.setWebDAVOptions(options.serve_image)
+        c.setIsADCSAttack(options.adcs)
+        c.setADCSOptions(options.template)
 
         if server is HTTPRelayServer:
             c.setListeningPort(options.http_port)
@@ -318,6 +322,11 @@ if __name__ == '__main__':
     imapoptions.add_argument('-im','--imap-max', action='store',type=int, required=False,default=0, help='Max number of emails to dump '
         '(0 = unlimited, default: no limit)')
 
+    # AD CS options
+    adcsoptions = parser.add_argument_group("AD CS attack options")
+    adcsoptions.add_argument('--adcs', action='store_true', required=False, help='Enable AD CS relay attack')
+    adcsoptions.add_argument('--template', action='store', metavar="TEMPLATE", required=False, default="Machine", help='AD CS template. If you are attacking Domain Controller or other windows server machine, default value should be suitable.')
+
     try:
        options = parser.parse_args()
     except Exception as e:
