@@ -1,22 +1,31 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
+#
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
 #
 # This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Description: Remote registry manipulation tool.
-#              The idea is to provide similar functionality as the REG.EXE Windows utility.
+# Description:
+#   Remote registry manipulation tool.
+#   The idea is to provide similar functionality as the REG.EXE Windows utility.
 #
-# e.g:
-#    ./reg.py Administrator:password@targetMachine query -keyName HKLM\\Software\\Microsoft\\WBEM -s
+#   e.g:
+#       ./reg.py Administrator:password@targetMachine query -keyName HKLM\\Software\\Microsoft\\WBEM -s
+#       ./reg.py Administrator:password@targetMachine add -keyName HKLM\\SYSTEM\\CurrentControlSet\\Control\\Lsa -v DisableRestrictedAdmin -vt REG_DWORD -vd 1
+#       ./reg.py Administrator:password@targetMachine add -keyName HKLM\\SYSTEM\\CurrentControlSet\\Services\\NewService
+#       ./reg.py Administrator:password@targetMachine add -keyName HKCR\\hlpfile\\DefaultIcon  -v '' -vd '\\SMBRelay\share'
+#       ./reg.py Administrator:password@targetMachine delete -keyName HKLM\\SYSTEM\\CurrentControlSet\\Control\\Lsa -v DisableRestrictedAdmin
 #
 # Author:
-#  Manuel Porto (@manuporto)
-#  Alberto Solino (@agsolino)
+#   Manuel Porto (@manuporto)
+#   Alberto Solino (@agsolino)
 #
-# Reference for: [MS-RRP]
+# Reference for:
+#   [MS-RRP]
 #
+
 from __future__ import division
 from __future__ import print_function
 import argparse
@@ -33,6 +42,7 @@ from impacket.examples.utils import parse_target
 from impacket.system_errors import ERROR_NO_MORE_ITEMS
 from impacket.structure import hexdump
 from impacket.smbconnection import SMBConnection
+from impacket.dcerpc.v5.dtypes import READ_CONTROL
 
 
 class RemoteOperations:
@@ -171,6 +181,10 @@ class RegHandler:
 
             if self.__action == 'QUERY':
                 self.query(dce, self.__options.keyName)
+            elif self.__action == 'ADD':
+                self.add(dce, self.__options.keyName)
+            elif self.__action == 'DELETE':
+                self.delete(dce, self.__options.keyName)
             else:
                 logging.error('Method %s not implemented yet!' % self.__action)
         except (Exception, KeyboardInterrupt) as e:
@@ -182,23 +196,7 @@ class RegHandler:
                 self.__remoteOps.finish()
 
     def query(self, dce, keyName):
-        # Let's strip the root key
-        try:
-            rootKey = keyName.split('\\')[0]
-            subKey = '\\'.join(keyName.split('\\')[1:])
-        except Exception:
-            raise Exception('Error parsing keyName %s' % keyName)
-
-        if rootKey.upper() == 'HKLM':
-            ans = rrp.hOpenLocalMachine(dce)
-        elif rootKey.upper() == 'HKU':
-            ans = rrp.hOpenCurrentUser(dce)
-        elif rootKey.upper() == 'HKCR':
-            ans = rrp.hOpenClassesRoot(dce)
-        else:
-            raise Exception('Invalid root key %s ' % rootKey)
-
-        hRootKey = ans['phKey']
+        hRootKey, subKey = self.__strip_root_key(dce, keyName)
 
         ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
                                    samDesired=rrp.MAXIMUM_ALLOWED | rrp.KEY_ENUMERATE_SUB_KEYS | rrp.KEY_QUERY_VALUE)
@@ -227,6 +225,187 @@ class RegHandler:
                     # ans5 = rrp.hBaseRegGetVersion(rpc, ans2['phkResult'])
                     # ans3 = rrp.hBaseRegEnumKey(rpc, ans2['phkResult'], 0)
 
+    def add(self, dce, keyName):
+        hRootKey, subKey = self.__strip_root_key(dce, keyName)
+
+        # READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY should be equal to KEY_WRITE (0x20006)
+        if self.__options.v is None: # Try to create subkey
+            subKeyCreate = subKey
+            subKey = '\\'.join(subKey.split('\\')[:-1])
+
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY)
+
+            # Should I use ans2?
+
+            ans3 = rrp.hBaseRegCreateKey(
+                dce, hRootKey, subKeyCreate,
+                samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY
+            )
+            if ans3['ErrorCode'] == 0:
+                print('Successfully set subkey %s' % (
+                    keyName
+                ))
+            else:
+                print('Error 0x%08x while creating subkey %s' % (
+                    ans3['ErrorCode'], keyName
+                ))
+
+        else: # Try to set value of key
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY)
+
+
+            dwType = getattr(rrp, self.__options.vt, None)
+
+            if dwType is None or not self.__options.vt.startswith('REG_'):
+                raise Exception('Error parsing value type %s' % self.__options.vt)
+
+            #Fix (?) for packValue function
+            if dwType in (
+                rrp.REG_DWORD, rrp.REG_DWORD_BIG_ENDIAN, rrp.REG_DWORD_LITTLE_ENDIAN,
+                rrp.REG_QWORD, rrp.REG_QWORD_LITTLE_ENDIAN
+            ):
+                valueData = int(self.__options.vd)
+            else:
+                valueData = self.__options.vd
+
+            ans3 = rrp.hBaseRegSetValue(
+                dce, ans2['phkResult'], self.__options.v, dwType, valueData
+            )
+
+            if ans3['ErrorCode'] == 0:
+                print('Successfully set key %s\\%s of type %s to value %s' % (
+                    keyName, self.__options.v, self.__options.vt, valueData
+                ))
+            else:
+                print('Error 0x%08x while setting key %s\\%s of type %s to value %s' % (
+                    ans3['ErrorCode'], keyName, self.__options.v, self.__options.vt, valueData
+                ))
+
+    def delete(self, dce, keyName):
+        hRootKey, subKey = self.__strip_root_key(dce, keyName)
+
+        # READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY should be equal to KEY_WRITE (0x20006)
+        if self.__options.v is None and not self.__options.va and not self.__options.ve: # Try to delete subkey
+            subKeyDelete = subKey
+            subKey = '\\'.join(subKey.split('\\')[:-1])
+
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY)
+
+            # Should I use ans2?
+            try:
+                ans3 = rrp.hBaseRegDeleteKey(
+                    dce, hRootKey, subKeyDelete,
+                )
+            except rpcrt.DCERPCException as e:
+                if e.error_code == 5:
+                    #TODO: Check if DCERPCException appears only because of existing subkeys
+                    print('Cannot delete key %s. Possibly it contains subkeys or insufficient privileges' % keyName)
+                    return
+                else:
+                    raise
+            except Exception as e:
+                logging.error('Unhandled exception while hBaseRegDeleteKey')
+                return
+
+            if ans3['ErrorCode'] == 0:
+                print('Successfully deleted subkey %s' % (
+                    keyName
+                ))
+            else:
+                print('Error 0x%08x while deleting subkey %s' % (
+                    ans3['ErrorCode'], keyName
+                ))
+
+        elif self.__options.v: # Delete single value
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY)
+
+            ans3 = rrp.hBaseRegDeleteValue(
+                dce, ans2['phkResult'], self.__options.v
+            )
+
+            if ans3['ErrorCode'] == 0:
+                print('Successfully deleted key %s\\%s' % (
+                    keyName, self.__options.v
+                ))
+            else:
+                print('Error 0x%08x while deleting key %s\\%s' % (
+                    ans3['ErrorCode'], keyName, self.__options.v
+                ))
+
+        elif self.__options.ve:
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=READ_CONTROL | rrp.KEY_SET_VALUE | rrp.KEY_CREATE_SUB_KEY)
+
+            ans3 = rrp.hBaseRegDeleteValue(
+                dce, ans2['phkResult'], ''
+            )
+
+            if ans3['ErrorCode'] == 0:
+                print('Successfully deleted value %s\\%s' % (
+                    keyName, 'Default'
+                ))
+            else:
+                print('Error 0x%08x while deleting value %s\\%s' % (
+                    ans3['ErrorCode'], keyName, self.__options.v
+                ))
+
+        elif self.__options.va:
+            ans2 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=rrp.MAXIMUM_ALLOWED | rrp.KEY_ENUMERATE_SUB_KEYS)
+            i = 0
+            allSubKeys = []
+            while True:
+                try:
+                    ans3 = rrp.hBaseRegEnumValue(dce, ans2['phkResult'], i)
+                    lp_value_name = ans3['lpValueNameOut'][:-1]
+                    allSubKeys.append(lp_value_name)
+                    i += 1
+                except rrp.DCERPCSessionError as e:
+                    if e.get_error_code() == ERROR_NO_MORE_ITEMS:
+                        break
+
+            ans4 = rrp.hBaseRegOpenKey(dce, hRootKey, subKey,
+                                       samDesired=rrp.MAXIMUM_ALLOWED | rrp.KEY_ENUMERATE_SUB_KEYS)
+            for subKey in allSubKeys:
+                try:
+                    ans5 = rrp.hBaseRegDeleteValue(
+                        dce, ans4['phkResult'], subKey
+                    )
+                    if ans5['ErrorCode'] == 0:
+                        print('Successfully deleted value %s\\%s' % (
+                            keyName, subKey
+                        ))
+                    else:
+                        print('Error 0x%08x in deletion of value %s\\%s' % (
+                            ans5['ErrorCode'], keyName, subKey
+                        ))
+                except Exception as e:
+                    print('Unhandled error %s in deletion of value %s\\%s' % (
+                        str(e), keyName, subKey
+                    ))
+
+    def __strip_root_key(self, dce, keyName):
+        # Let's strip the root key
+        try:
+            rootKey = keyName.split('\\')[0]
+            subKey = '\\'.join(keyName.split('\\')[1:])
+        except Exception:
+            raise Exception('Error parsing keyName %s' % keyName)
+        if rootKey.upper() == 'HKLM':
+            ans = rrp.hOpenLocalMachine(dce)
+        elif rootKey.upper() == 'HKU':
+            ans = rrp.hOpenCurrentUser(dce)
+        elif rootKey.upper() == 'HKCR':
+            ans = rrp.hOpenClassesRoot(dce)
+        else:
+            raise Exception('Invalid root key %s ' % rootKey)
+        hRootKey = ans['phKey']
+        return hRootKey, subKey
+
     def __print_key_values(self, rpc, keyHandler):
         i = 0
         while True:
@@ -325,7 +504,7 @@ if __name__ == '__main__':
     query_parser.add_argument('-keyName', action='store', required=True,
                               help='Specifies the full path of the subkey. The '
                                    'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
-                                   ' HKU.')
+                                   ' HKU, HKCR.')
     query_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
                            'value name that is to be queried. If omitted, all value names for keyName are returned. ')
     query_parser.add_argument('-ve', action='store_true', default=False, required=False, help='Queries for the default '
@@ -334,10 +513,30 @@ if __name__ == '__main__':
                                                                              'names recursively.')
 
     # An add command
-    # add_parser = subparsers.add_parser('add', help='Adds a new subkey or entry to the registry')
+    add_parser = subparsers.add_parser('add', help='Adds a new subkey or entry to the registry')
+    add_parser.add_argument('-keyName', action='store', required=True,
+                              help='Specifies the full path of the subkey. The '
+                                   'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
+                                   ' HKU, HKCR.')
+    add_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
+                           'value name that is to be set.')
+    add_parser.add_argument('-vt', action='store', metavar="VALUETYPE", required=False, help='Specifies the registry '
+                           'type name that is to be set. Default is REG_SZ. Valid types are: REG_NONE, REG_SZ, REG_EXPAND_SZ, '
+                           'REG_BINARY, REG_DWORD, REG_DWORD_BIG_ENDIAN, REG_LINK, REG_MULTI_SZ, REG_QWORD',
+                            default='REG_SZ')
+    add_parser.add_argument('-vd', action='store', metavar="VALUEDATA", required=False, help='Specifies the registry '
+                           'value data that is to be set.', default='')
 
     # An delete command
-    # delete_parser = subparsers.add_parser('delete', help='Deletes a subkey or entries from the registry')
+    delete_parser = subparsers.add_parser('delete', help='Deletes a subkey or entries from the registry')
+    delete_parser.add_argument('-keyName', action='store', required=True,
+                              help='Specifies the full path of the subkey. The '
+                                   'keyName must include a valid root key. Valid root keys for the local computer are: HKLM,'
+                                   ' HKU, HKCR.')
+    delete_parser.add_argument('-v', action='store', metavar="VALUENAME", required=False, help='Specifies the registry '
+                           'value name that is to be deleted.')
+    delete_parser.add_argument('-va', action='store_true', required=False, help='Delete all values under this key.')
+    delete_parser.add_argument('-ve', action='store_true', required=False, help='Delete the value of empty value name (Default).')
 
     # A copy command
     # copy_parser = subparsers.add_parser('copy', help='Copies a registry entry to a specified location in the remote '
