@@ -1,18 +1,23 @@
 #!/usr/bin/python
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2021 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Description: DCE/RPC SAMR dumper.
+# Description:
+#   DCE/RPC SAMR dumper.
 #
 # Author:
-#  Javier Kohen <jkohen@coresecurity.com>
-#  Alberto Solino (@agsolino)
+#   Javier Kohen
+#   Alberto Solino (@agsolino)
 #
 # Reference for:
-#  DCE/RPC for SAMR
+#   DCE/RPC for SAMR
+#
+
 from __future__ import division
 from __future__ import print_function
 import sys
@@ -22,11 +27,11 @@ import codecs
 
 from datetime import datetime
 from impacket.examples import logger
+from impacket.examples.utils import parse_target
 from impacket import version
 from impacket.nt_errors import STATUS_MORE_ENTRIES
 from impacket.dcerpc.v5 import transport, samr
 from impacket.dcerpc.v5.rpcrt import DCERPCException
-from impacket.smb import SMB_DIALECT
 
 class ListUsersException(Exception):
     pass
@@ -70,8 +75,6 @@ class SAMRDump:
         rpctransport.set_dport(self.__port)
         rpctransport.setRemoteHost(remoteHost)
 
-        if hasattr(rpctransport,'preferred_dialect'):
-            rpctransport.preferred_dialect(SMB_DIALECT)
         if hasattr(rpctransport, 'set_credentials'):
             # This method exists only for selected protocol sequences.
             rpctransport.set_credentials(self.__username, self.__password, self.__domain, self.__lmhash,
@@ -190,8 +193,6 @@ class SAMRDump:
 
 # Process command-line arguments.
 if __name__ == '__main__':
-    # Init the example's logger theme
-    logger.init()
     # Explicitly changing the stdout encoding format
     if sys.stdout.encoding is None:
         # Output is redirected to a file
@@ -203,6 +204,7 @@ if __name__ == '__main__':
 
     parser.add_argument('target', action='store', help='[[domain/]username[:password]@]<targetName or address>')
     parser.add_argument('-csv', action='store_true', help='Turn CSV output')
+    parser.add_argument('-ts', action='store_true', help='Adds timestamp to every logging output')
     parser.add_argument('-debug', action='store_true', help='Turn DEBUG output ON')
 
     group = parser.add_argument_group('connection')
@@ -231,21 +233,18 @@ if __name__ == '__main__':
 
     options = parser.parse_args()
 
+    # Init the example's logger theme
+    logger.init(options.ts)
+
     if options.debug is True:
         logging.getLogger().setLevel(logging.DEBUG)
+        # Print the Library's installation path
+        logging.debug(version.getInstallationPath())
     else:
         logging.getLogger().setLevel(logging.INFO)
 
-    import re
+    domain, username, password, remoteName = parse_target(options.target)
 
-    domain, username, password, remoteName = re.compile('(?:(?:([^/@:]*)/)?([^@:]*)(?::([^@]*))?@)?(.*)').match(
-        options.target).groups('')
-
-    #In case the password contains '@'
-    if '@' in remoteName:
-        password = password + '@' + remoteName.rpartition('@')[0]
-        remoteName = remoteName.rpartition('@')[2]
-        
     if domain is None:
         domain = ''
 
