@@ -1,11 +1,11 @@
-# SECUREAUTH LABS. Copyright 2018 SecureAuth Corporation. All rights reserved.
+# Impacket - Collection of Python classes for working with network protocols.
 #
-# This software is provided under under a slightly modified version
+# SECUREAUTH LABS. Copyright (C) 2019 SecureAuth Corporation. All rights reserved.
+#
+# This software is provided under a slightly modified version
 # of the Apache Software License. See the accompanying LICENSE file
 # for more information.
 #
-# Author: Alberto Solino (@agsolino)
-#
 # Description:
 #   [MS-SAMR] Interface implementation
 #
@@ -14,10 +14,15 @@
 #   at https://github.com/SecureAuthCorp/impacket/tree/master/tests/SMB_RPC
 #
 #   Some calls have helper functions, which makes it even easier to use.
-#   They are located at the end of this file. 
+#   They are located at the end of this file.
 #   Helper functions start with "h"<name of the call>.
-#   There are test cases for them too. 
+#   There are test cases for them too.
 #
+# Author:
+#   Alberto Solino (@agsolino)
+#
+from __future__ import division
+from __future__ import print_function
 from binascii import unhexlify
 
 from impacket.dcerpc.v5.ndr import NDRCALL, NDR, NDRSTRUCT, NDRUNION, NDRPOINTER, NDRUniConformantArray, \
@@ -30,6 +35,11 @@ from impacket.uuid import uuidtup_to_bin
 from impacket.dcerpc.v5.enum import Enum
 from impacket.structure import Structure
 
+import struct
+import os
+from hashlib import md5
+from Cryptodome.Cipher import ARC4
+
 MSRPC_UUID_SAMR   = uuidtup_to_bin(('12345778-1234-ABCD-EF00-0123456789AC', '1.0'))
 
 class DCERPCSessionError(DCERPCException):
@@ -38,9 +48,9 @@ class DCERPCSessionError(DCERPCException):
 
     def __str__( self ):
         key = self.error_code
-        if nt_errors.ERROR_MESSAGES.has_key(key):
+        if key in nt_errors.ERROR_MESSAGES:
             error_msg_short = nt_errors.ERROR_MESSAGES[key][0]
-            error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1] 
+            error_msg_verbose = nt_errors.ERROR_MESSAGES[key][1]
             return 'SAMR SessionError: code: 0x%x - %s - %s' % (self.error_code, error_msg_short, error_msg_verbose)
         else:
             return 'SAMR SessionError: unknown error code: 0x%x' % self.error_code
@@ -298,17 +308,18 @@ class RPC_STRING(NDRSTRUCT):
     )
 
     def dump(self, msg = None, indent = 0):
-        if msg is None: msg = self.__class__.__name__
+        if msg is None:
+            msg = self.__class__.__name__
         if msg != '':
-            print "%s" % msg,
+            print("%s" % msg, end=' ')
         # Here just print the data
-        print " %r" % (self['Data']),
+        print(" %r" % (self['Data']), end=' ')
 
 class PRPC_STRING(NDRPOINTER):
     referent = (
         ('Data', RPC_STRING),
     )
- 
+
 # 2.2.2.2 OLD_LARGE_INTEGER
 class OLD_LARGE_INTEGER(NDRSTRUCT):
     structure = (
@@ -350,7 +361,7 @@ class RPC_SHORT_BLOB(NDRSTRUCT):
 # 2.2.3.2 SAMPR_HANDLE
 class SAMPR_HANDLE(NDRSTRUCT):
     structure =  (
-        ('Data','20s=""'),
+        ('Data','20s=b""'),
     )
     def getAlignment(self):
         if self._isNDR64 is True:
@@ -361,7 +372,7 @@ class SAMPR_HANDLE(NDRSTRUCT):
 # 2.2.3.3 ENCRYPTED_LM_OWF_PASSWORD, ENCRYPTED_NT_OWF_PASSWORD
 class ENCRYPTED_LM_OWF_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Data', '16s=""'),
+        ('Data', '16s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -495,7 +506,7 @@ class GROUP_MEMBERSHIP_ARRAY(NDRUniConformantArray):
 class PGROUP_MEMBERSHIP_ARRAY(NDRPOINTER):
     referent = (
         ('Data',GROUP_MEMBERSHIP_ARRAY),
-    ) 
+    )
 
 # 2.2.3.13 SAMPR_GET_GROUPS_BUFFER
 class SAMPR_GET_GROUPS_BUFFER(NDRSTRUCT):
@@ -720,7 +731,7 @@ class SAMPR_GROUP_ADM_COMMENT_INFORMATION(NDRSTRUCT):
 # 2.2.5.6 GROUP_INFORMATION_CLASS
 class GROUP_INFORMATION_CLASS(NDRENUM):
     class enumItems(Enum):
-        GroupGeneralInformation      = 1 
+        GroupGeneralInformation      = 1
         GroupNameInformation         = 2
         GroupAttributeInformation    = 3
         GroupAdminCommentInformation = 4
@@ -775,7 +786,7 @@ class SAMPR_ALIAS_INFO_BUFFER(NDRUNION):
         ALIAS_INFORMATION_CLASS.AliasNameInformation         : ('Name', SAMPR_ALIAS_NAME_INFORMATION),
         ALIAS_INFORMATION_CLASS.AliasAdminCommentInformation : ('AdminComment', SAMPR_ALIAS_ADM_COMMENT_INFORMATION),
     }
- 
+
 class PSAMPR_ALIAS_INFO_BUFFER(NDRPOINTER):
     referent = (
         ('Data', SAMPR_ALIAS_INFO_BUFFER),
@@ -816,7 +827,8 @@ class SAMPR_LOGON_HOURS(NDRSTRUCT):
     )
 
     def getData(self, soFar = 0):
-        self['UnitsPerWeek'] = len(self['LogonHours']) * 8 
+        if self['LogonHours'] != 0:
+            self['UnitsPerWeek'] = len(self['LogonHours']) * 8
         return NDR.getData(self, soFar)
 
 # 2.2.7.6 SAMPR_USER_ALL_INFORMATION
@@ -990,7 +1002,7 @@ class SAMPR_USER_LOGON_HOURS_INFORMATION(NDRSTRUCT):
 # 2.2.7.21 SAMPR_ENCRYPTED_USER_PASSWORD
 class SAMPR_USER_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Buffer', '512s=""'),
+        ('Buffer', '512s=b""'),
         ('Length', ULONG),
     )
     def getAlignment(self):
@@ -999,7 +1011,7 @@ class SAMPR_USER_PASSWORD(NDRSTRUCT):
 
 class SAMPR_ENCRYPTED_USER_PASSWORD(NDRSTRUCT):
     structure = (
-        ('Buffer', '516s=""'),
+        ('Buffer', '516s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1012,7 +1024,7 @@ class PSAMPR_ENCRYPTED_USER_PASSWORD(NDRPOINTER):
 # 2.2.7.22 SAMPR_ENCRYPTED_USER_PASSWORD_NEW
 class SAMPR_ENCRYPTED_USER_PASSWORD_NEW(NDRSTRUCT):
     structure = (
-        ('Buffer', '522s=""'),
+        ('Buffer', '532s=b""'),
     )
     def getAlignment(self):
         return 1
@@ -1109,7 +1121,7 @@ class SAMPR_USER_INFO_BUFFER(NDRUNION):
         USER_INFORMATION_CLASS.UserInternal4InformationNew: ('Internal4New', SAMPR_USER_INTERNAL4_INFORMATION_NEW),
         USER_INFORMATION_CLASS.UserInternal5InformationNew: ('Internal5New', SAMPR_USER_INTERNAL5_INFORMATION_NEW),
     }
- 
+
 class PSAMPR_USER_INFO_BUFFER(NDRPOINTER):
     referent = (
         ('Data', SAMPR_USER_INFO_BUFFER),
@@ -1117,8 +1129,8 @@ class PSAMPR_USER_INFO_BUFFER(NDRPOINTER):
 
 class PSAMPR_SERVER_NAME2(NDRPOINTER):
     referent = (
-        ('Data', '4s=""'),
-    ) 
+        ('Data', '4s=b""'),
+    )
 
 # 2.2.8.2 SAMPR_DOMAIN_DISPLAY_USER
 class SAMPR_DOMAIN_DISPLAY_USER(NDRSTRUCT):
@@ -1218,21 +1230,21 @@ class SAMPR_DOMAIN_DISPLAY_MACHINE_BUFFER(NDRSTRUCT):
         ('EntriesRead', ULONG),
         ('Buffer', PSAMPR_DOMAIN_DISPLAY_MACHINE_ARRAY),
     )
- 
+
 # 2.2.8.9 SAMPR_DOMAIN_DISPLAY_GROUP_BUFFER
 class SAMPR_DOMAIN_DISPLAY_GROUP_BUFFER(NDRSTRUCT):
     structure = (
         ('EntriesRead', ULONG),
         ('Buffer', PSAMPR_DOMAIN_DISPLAY_GROUP_ARRAY),
     )
- 
+
 # 2.2.8.10 SAMPR_DOMAIN_DISPLAY_OEM_USER_BUFFER
 class SAMPR_DOMAIN_DISPLAY_OEM_USER_BUFFER(NDRSTRUCT):
     structure = (
         ('EntriesRead', ULONG),
         ('Buffer', PSAMPR_DOMAIN_DISPLAY_OEM_USER_ARRAY),
     )
- 
+
 # 2.2.8.11 SAMPR_DOMAIN_DISPLAY_OEM_GROUP_BUFFER
 class SAMPR_DOMAIN_DISPLAY_OEM_GROUP_BUFFER(NDRSTRUCT):
     structure = (
@@ -2418,12 +2430,13 @@ OPNUMS = {
 # HELPER FUNCTIONS
 ################################################################################
 
-def hSamrConnect5(dce, serverName='\x00', desiredAccess=MAXIMUM_ALLOWED, inVersion=1):
+def hSamrConnect5(dce, serverName='\x00', desiredAccess=MAXIMUM_ALLOWED, inVersion=1, revision=3):
     request = SamrConnect5()
     request['ServerName'] = serverName
     request['DesiredAccess'] = desiredAccess
     request['InVersion'] = inVersion
     request['InRevisionInfo']['tag'] = inVersion
+    request['InRevisionInfo']['V1']['Revision'] = revision
     return dce.request(request)
 
 def hSamrConnect4(dce, serverName='\x00', desiredAccess=MAXIMUM_ALLOWED, clientRevision=2):
@@ -2725,22 +2738,45 @@ def hSamrGetAliasMembership(dce, domainHandle, sidArray):
     request['SidArray']['Count'] = len(sidArray['Sids'])
     return dce.request(request)
 
-def hSamrChangePasswordUser(dce, userHandle, oldPassword, newPassword):
+def hSamrChangePasswordUser(dce, userHandle, oldPassword, newPassword, oldPwdHashNT='', newPwdHashLM='', newPwdHashNT=''):
     request = SamrChangePasswordUser()
     request['UserHandle'] = userHandle
 
     from impacket import crypto, ntlm
 
-    oldPwdHashNT = ntlm.NTOWFv1(oldPassword)
-    newPwdHashNT = ntlm.NTOWFv1(newPassword)
-    newPwdHashLM = ntlm.LMOWFv1(newPassword)
+    if oldPwdHashNT == '':
+        oldPwdHashNT = ntlm.NTOWFv1(oldPassword)
+    else:
+        # Let's convert the hashes to binary form, if not yet
+        try:
+            oldPwdHashNT = unhexlify(oldPwdHashNT)
+        except:
+            pass
+
+    if newPwdHashLM == '':
+        newPwdHashLM = ntlm.LMOWFv1(newPassword)
+    else:
+        # Let's convert the hashes to binary form, if not yet
+        try:
+            newPwdHashLM = unhexlify(newPwdHashLM)
+        except:
+            pass
+
+    if newPwdHashNT == '':
+        newPwdHashNT = ntlm.NTOWFv1(newPassword)
+    else:
+        # Let's convert the hashes to binary form, if not yet
+        try:
+            newPwdHashNT = unhexlify(newPwdHashNT)
+        except:
+            pass
 
     request['LmPresent'] = 0
     request['OldLmEncryptedWithNewLm'] = NULL
     request['NewLmEncryptedWithOldLm'] = NULL
     request['NtPresent'] = 1
     request['OldNtEncryptedWithNewNt'] = crypto.SamEncryptNTLMHash(oldPwdHashNT, newPwdHashNT)
-    request['NewNtEncryptedWithOldNt'] = crypto.SamEncryptNTLMHash(newPwdHashNT, oldPwdHashNT) 
+    request['NewNtEncryptedWithOldNt'] = crypto.SamEncryptNTLMHash(newPwdHashNT, oldPwdHashNT)
     request['NtCrossEncryptionPresent'] = 0
     request['NewNtEncryptedWithNewLm'] = NULL
     request['LmCrossEncryptionPresent'] = 1
@@ -2769,24 +2805,22 @@ def hSamrUnicodeChangePasswordUser2(dce, serverName='\x00', userName='', oldPass
             oldPwdHashLM = unhexlify(oldPwdHashLM)
         except:
             pass
-        try: 
+        try:
             oldPwdHashNT = unhexlify(oldPwdHashNT)
         except:
             pass
 
     newPwdHashNT = ntlm.NTOWFv1(newPassword)
-    newPwdHashLM = ntlm.LMOWFv1(newPassword)
-
 
     samUser = SAMPR_USER_PASSWORD()
     try:
-        samUser['Buffer'] = 'A'*(512-len(newPassword)*2) + newPassword.encode('utf-16le')
+        samUser['Buffer'] = b'A'*(512-len(newPassword)*2) + newPassword.encode('utf-16le')
     except UnicodeDecodeError:
         import sys
-        samUser['Buffer'] = 'A'*(512-len(newPassword)*2) + newPassword.decode(sys.getfilesystemencoding()).encode('utf-16le')
+        samUser['Buffer'] = b'A'*(512-len(newPassword)*2) + newPassword.decode(sys.getfilesystemencoding()).encode('utf-16le')
 
     samUser['Length'] = len(newPassword)*2
-    pwdBuff = str(samUser)
+    pwdBuff = samUser.getData()
 
     rc4 = ARC4.new(oldPwdHashNT)
     encBuf = rc4.encrypt(pwdBuff)
@@ -2877,3 +2911,45 @@ def hSamrLookupIdsInDomain(dce, domainHandle, ids):
 
     return dce.request(request)
 
+def hSamrSetPasswordInternal4New(dce, userHandle, password):
+    request = SamrSetInformationUser2()
+    request['UserHandle'] = userHandle
+    request['UserInformationClass'] = USER_INFORMATION_CLASS.UserInternal4InformationNew
+    request['Buffer']['tag'] =  USER_INFORMATION_CLASS.UserInternal4InformationNew
+    request['Buffer']['Internal4New']['I1']['WhichFields'] = 0x01000000 | 0x08000000
+
+    request['Buffer']['Internal4New']['I1']['UserName'] = NULL
+    request['Buffer']['Internal4New']['I1']['FullName'] = NULL
+    request['Buffer']['Internal4New']['I1']['HomeDirectory'] = NULL
+    request['Buffer']['Internal4New']['I1']['HomeDirectoryDrive'] = NULL
+    request['Buffer']['Internal4New']['I1']['ScriptPath'] = NULL
+    request['Buffer']['Internal4New']['I1']['ProfilePath'] = NULL
+    request['Buffer']['Internal4New']['I1']['AdminComment'] = NULL
+    request['Buffer']['Internal4New']['I1']['WorkStations'] = NULL
+    request['Buffer']['Internal4New']['I1']['UserComment'] = NULL
+    request['Buffer']['Internal4New']['I1']['Parameters'] = NULL
+    request['Buffer']['Internal4New']['I1']['LmOwfPassword']['Buffer'] = NULL
+    request['Buffer']['Internal4New']['I1']['NtOwfPassword']['Buffer'] = NULL
+    request['Buffer']['Internal4New']['I1']['PrivateData'] = NULL
+    request['Buffer']['Internal4New']['I1']['SecurityDescriptor']['SecurityDescriptor'] = NULL
+    request['Buffer']['Internal4New']['I1']['LogonHours']['LogonHours'] = NULL
+    request['Buffer']['Internal4New']['I1']['PasswordExpired'] = 1
+
+    #crypto
+    pwdbuff = password.encode("utf-16le")
+    bufflen = len(pwdbuff)
+    pwdbuff = pwdbuff.rjust(512, b'\0')
+    pwdbuff += struct.pack('<I', bufflen)
+    salt = os.urandom(16)
+    session_key = dce.get_rpc_transport().get_smb_connection().getSessionKey()
+    keymd = md5()
+    keymd.update(salt)
+    keymd.update(session_key)
+    key = keymd.digest()
+
+    cipher = ARC4.new(key)
+    buffercrypt = cipher.encrypt(pwdbuff) + salt
+
+
+    request['Buffer']['Internal4New']['UserPassword']['Buffer'] = buffercrypt
+    return dce.request(request)
